import javax.swing.*;

import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.io.*;
/**
 * DigitalCircuit class creates a new circuit to simulate
 * @author Marcio B
 *
 */
public class DigitalCircuit{
  private JPanel panel;
  private Simulator simulator;
  private ArrayList<Drawable> components;
  private JFrame frame;
  /**
   * Constructor of the class
   * @param f JFrame
   */
  public DigitalCircuit(JFrame f){
    simulator = new Simulator();
    components = new ArrayList<Drawable>();
    frame = f;
   
  }
  public void setPanel(JPanel p){
    panel = p;
  }
  public void simulate(){
    simulator.simulate();
  }

  public Drawable find(Point p) {
     for (Drawable component : components)
       if (component.contains(p))
          return component;
     return null;
  }
  public void delete(Drawable comp){
	components.remove(comp);  
  }

  public void removeAll(){
	  components.clear();
	  panel.repaint();
  }
  /**
   * Sets the delay of a gate
   * @param comp
   */
  public void setGateDelay(Drawable comp){
	  String gateString = comp.getClass().getName();
	  float d = Float.parseFloat(
				JOptionPane.showInputDialog(
						null,"Enter value delay for " + gateString.substring(1,gateString.length()-4)+" :",
						"value between 0 : 200"));
	  while((d < 0) | (d > 200)){
		  d = Float.parseFloat(
					JOptionPane.showInputDialog(
							null,"Wrong input !! \n Enter delay for " + gateString.substring(1,gateString.length()-4) + " again",
							"value between 0 : 200"));
	  }
		 
	  comp.setGDelay(d);
  }

  public GOutSignal findSource(Rectangle2D rec) {
     for (Drawable component : components) 
       if (component instanceof GOutSignal) {
          GOutSignal source = (GOutSignal)component;
          if (source.getGOutput().intersects(rec))
            return source;
       }
     return null;
  }
  public GInSignal findDrain(Rectangle2D rec) {
     for (Drawable component : components) 
       if (component instanceof GInSignal) {
          GInSignal drain = (GInSignal)component;
          if (drain.getGInput().intersects(rec))
            return drain;
       }
     return null;
  }

  public void paintComponent(Graphics2D g) {
    for (Drawable c: components) 
      c.draw(g);
  }
  /**
   * Charges a new .dc files.
   * Takes list of components and reconnect wires.
   * @param data FileReader
   */
  public void addOpenFile(FileReader data){
	this.removeAll();
	OpenFile file = new OpenFile(data, simulator, this, frame);  
	file.addComp();
	components = file.getComponents();
	panel.repaint();
	for(Drawable c: components){
		if(c instanceof GWire){
			((GWire) c).setReleased();
			((GWire) c).forceConnect();
			panel.repaint();
		}	
	}	
	panel.repaint();
	
  }
  /**
   * Creates a new file to save
   * @param data PrintStream
   */
  public void addSaveFile(PrintStream data){
	  SaveFile file = new SaveFile(components, data);
	  file.write();
  }
 
  public void addInverter() {
    components.add(new GInverterGate(new Point2D.Double(200f, 200f), simulator));
    panel.repaint();
  }
  public void addOrGate() {
    components.add(new GOrGate(new Point2D.Double(200f, 200f), simulator));
    panel.repaint();
  }
  
  public void addNorGate() {
	    components.add(new GNorGate(new Point2D.Double(200f, 200f), simulator));
	    panel.repaint();
	  }

  public void addNandGate() {
	    components.add(new GNandGate(new Point2D.Double(200f, 200f), simulator));
	    panel.repaint();
	  }
  public void addXorGate() {
	    components.add(new GXorGate(new Point2D.Double(200f, 200f), simulator));
	    panel.repaint();
	  }
  public void addAndGate() {
	    components.add(new GAndGate(new Point2D.Double(200f, 200f), simulator));
	    panel.repaint();
	  }
  
  public void addWire() {
    components.add(new GWire(new Point2D.Double(200f, 200f), this));
    panel.repaint();
  }
  
  public void addSource(Scanner data) {
    components.add(new GSource(new Point2D.Double(200f, 200f), data, simulator));
    panel.repaint();
  }

  //metodo para crear el grafico
  public void addMeter(PrintStream data, JFrame frame) {
	    components.add(new GMeter(new Point2D.Double(200f, 200f), data, simulator, frame, new GraphFrame()));
	    panel.repaint();
  }
  /**
   * Changes the selected color with JDialog
   */
  public void changeSelColor(){ 
	    Color selcolor = JColorChooser.showDialog(panel,"Choose Selected Color", Color.BLACK);
	      
	  for(Drawable c:components){
		  c.setSelColor(selcolor);
		  c.setSelected();
		  c.setReleased();
	  }
	  panel.repaint();	
  }
  /**
   * Show graph of meter components
   * @param c
   */
  public void showGraph(Drawable c){
	  c.graph();
  }
  
  /**
   * Changes the released color
   * with a JDialog
   */
  public void changeRelColor(){
	  Color relcolor = JColorChooser.showDialog(panel,"Choose Release Color", Color.BLACK);
	  
	  for(Drawable c:components){
		  c.setRelColor(relcolor);
		  c.setReleased();
		  }
	  panel.repaint();
  }
	
}
