import java.awt.*;
import java.awt.geom.*;

/**
 * GAndGate class Generate visual  object
 * for Swing interface
 * 
 * @author Marcio B & Antonio M
 * @version v2
 */
class GAndGate extends AndGate implements Drawable, GOutSignal, GInSignal {
   private static final Path2D.Double polyline = new Path2D.Double(Path2D.WIND_EVEN_ODD,10);
                     
   private static final Rectangle2D.Double input= new Rectangle2D.Double(-5,15,10,10);
   private static final Rectangle2D.Double output= new Rectangle2D.Double(95,15,10,10);
  
   static {  // static initialization block. 	
      polyline.moveTo(20,40);
      polyline.lineTo(20,0);
      polyline.moveTo(20,20);
      polyline.lineTo(0,20);
      polyline.moveTo(20,40);
      polyline.curveTo(86,40, 86,0, 20,0);
      polyline.moveTo(70,20);
      polyline.lineTo(100,20);
   }
   private Color color = Color.BLUE;
   private Color selColor = Color.RED;
   private Color relColor = Color.BLUE;
   private Path2D.Double shape;
   private Path2D.Double in;
   private Path2D.Double out;
   private final BasicStroke stroke;
   /**
    * Constructor of the class
    * @param pos Point2D.Double
    * @param sim Simulator
    */
   public GAndGate(Point2D.Double pos, Simulator sim){
      super(sim);
      AffineTransform at = AffineTransform.getTranslateInstance(pos.x, pos.y);
      shape = (Path2D.Double) at.createTransformedShape(polyline);
      in = (Path2D.Double) at.createTransformedShape(input);
      out = (Path2D.Double) at.createTransformedShape(output);
      stroke = new BasicStroke(2);

   }
   
   /**
    * Draws the visual object for And
    * @param g Graphics2D
    */
   public void draw (Graphics2D g){
      g.setColor(color);
      g.setStroke(stroke);
      g.draw(shape);
      
      if (hasConnectedSource())
        g.fill(in);
      else
        g.draw(in);
      g.draw(out);
      getId();
      g.drawString("AND_"+ getId(), (float)(shape.getCurrentPoint().getX()-75), (float)shape.getCurrentPoint().getY()-22);
   }
   /**
    * Return a shape if p is there
    * @param p Point
    * @return shape Path2D.Double
    */
   public boolean contains (Point p){
     return shape.contains(p);
   }
   /**
    * Moves the object Meter
    * @param dx integer for x axis
    * @param dy integer for y axis
    */
   public void dragIn(int dx, int dy) {
      AffineTransform at = AffineTransform.getTranslateInstance(dx, dy);
      shape.transform(at);
      in.transform(at);
      out.transform(at);
      disconnectInput();
      disconnectAllOutput();
   }
   /**
    * Flips the Gate
    */
   public void flip() {
	   AffineTransform af = new AffineTransform();   
	   Rectangle2D box = shape.getBounds2D(); 
	   double x = box.getX()+box.getWidth()/2;

	   // Note: the transformations get applied IN REVERSE ORDER!
	   af.translate(x,0);
	   af.scale(-1, 1);
	   af.translate(-x, 0);          
	   shape.transform(af);
	   in.transform(af);
	   out.transform(af);

   }
   /**
    * Set object to selected
    */
   public void setSelected() {
     color = selColor;
   }
   /**
    * Set object to released
    */
   public void setReleased(){
     color = relColor;
   }
   /**
    * Set Selected color of object
    * @param c Color
    */
   public void setSelColor(Color c){
	   selColor = c;
   }
   /**
    * Set Released color of object
    * @param c Color
    */
   public void setRelColor(Color c){
	   relColor = c;
   }
   public Shape getGOutput() {
     return out;
   }
   public Shape getGInput() {
     return in;
   }
   /**
    * Set delay for And
    * @param d float
    */
   public void setGDelay(float d){
	   setDelay(d);
   }


@Override
public  Path2D.Double getPos() {
	// TODO Auto-generated method stub
	return shape;
}
@Override
public float getGDelay() {
	// TODO Auto-generated method stub
	return getDelay();
}

@Override
public int getIdDraw() {
	// TODO Auto-generated method stub
	return getId();
}

@Override
public void graph() {
	// TODO Auto-generated method stub
	
}
}