import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Path2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
//import java.io.OutputStream;
import java.io.PrintStream;

import javax.swing.JFrame;
/**
 * Class for meter visual object.
 * Encapsulates color behaviors and movements.
 * @author Marcio B & Antonio M
 *
 */
class GMeter extends Meter implements Drawable, GInSignal {
   private static final Path2D.Double polyline = new Path2D.Double(Path2D.WIND_EVEN_ODD,5);
   private static final Rectangle2D.Double input = new Rectangle2D.Double(-5,5,10,10);
   static {  // static initialization block. 
      
      polyline.moveTo(75,10);
      polyline.lineTo(75,40);
      polyline.lineTo(45,40);
      polyline.lineTo(45,-20);
      polyline.lineTo(75,-20);
      polyline.lineTo(75,10);
      polyline.moveTo(45,10);
      polyline.lineTo(0,10);
      //Those lines do a rectangle 
      polyline.moveTo(50, -15);
      polyline.lineTo(50,5);
      polyline.lineTo(70,5);
      polyline.lineTo(70,-15);
      polyline.lineTo(50,-15);
      // Those lines draws a sin() wave
      polyline.moveTo(53, 25);
      polyline.curveTo(53, 35, 60, 35, 60, 25);
      polyline.curveTo(60, 15, 67, 15, 67, 25);
   }
   private Color color = Color.BLUE;
   private Color selColor = Color.RED;
   private Color relColor = Color.BLUE;
   private Path2D.Double shape;
   private Path2D.Double in;
   private static Graphics2D g;
   private GraphFrame graph;
   private JFrame f;
   private static BasicStroke stroke;

   /**
    * Constructor for GMeter class
    * @param pos Point2D.Double
    * @param output PrintStream
    * @param sim Simulator
    */
   public GMeter(Point2D.Double pos, PrintStream output, Simulator sim, JFrame frame, GraphFrame gf){
      super(output, sim, g, gf);
      f = frame;
      AffineTransform at = AffineTransform.getTranslateInstance(pos.x, pos.y);
      shape = (Path2D.Double) at.createTransformedShape(polyline);
      in = (Path2D.Double) at.createTransformedShape(input);
      stroke = new BasicStroke(2);
      graph = gf;
      graph.doFrame(f, getId());
   }
   /**
    * Draws the visual object for Meter
    * @param g Graphics2D
    */
   public void draw (Graphics2D gd){
	  g = gd;
      g.setColor(color);
      g.setStroke(stroke);
      g.draw(shape);
      g.draw(in);
      getId();
      g.drawString("METER_"+ getId(), (float)(shape.getCurrentPoint().getX()-30), (float)shape.getCurrentPoint().getY()+50);
      //graph.paint(g);
   }
   /**
    * Return a shape if p is there
    * @param p Point
    * @return shape Path2D.Double
    */
   public boolean contains (Point p){
     return shape.contains(p);
   }
   /**
    * Moves the object Meter
    * @param dx integer for x axis
    * @param dy integer for y axis
    */
   public void dragIn(int dx, int dy) {
      AffineTransform at = AffineTransform.getTranslateInstance(dx, dy);
      shape.transform(at);
      in.transform(at);
   }
   /**
    * Flips the Meter.
    */
   public void flip() {
     AffineTransform af = new AffineTransform();   
     Rectangle2D box = shape.getBounds2D(); 
     double x = box.getX()+box.getWidth()/2;

	  // Note: the transformations get applied IN REVERSE ORDER!
	  af.translate(x,0);
	  af.scale(-1, 1);
	  af.translate(-x, 0);          
	  shape.transform(af);
      in.transform(af);
   }
   /**
    * Set object to selected
    */
   public void setSelected() {
     color = selColor;
   }
   /**
    * Set object to released
    */
   public void setReleased(){
     color = relColor;
   }
   /**
    * Set color for selected
    * @param c Color
    */
   public void setSelColor(Color c){
	   selColor = c;
   }
   /**
    * Set color for released
    * @param c Color
    */
   public void setRelColor(Color c){
	   relColor = c;
   }
   public Shape getGInput() {
	     return in;
   }
   /**
    * Set delay..
    * Not usable
    * @param d float
    */
   public void setGDelay(float d){
	
   }
   
@Override
public Path2D.Double getPos() {
	// TODO Auto-generated method stub
	return shape;
}
@Override
public float getGDelay() {
	// TODO Auto-generated method stub
	return 0;
}
@Override
public int getIdDraw() {
	// TODO Auto-generated method stub
	return getId();
}
@Override
public void graph() {
	// TODO Auto-generated method stub
    graph.doFrame(f, getId());
}
}