import java.awt.*;
import java.awt.geom.*;

/**
 * Class to draw a wire
 * @author Marcio B
 *
 */
public class GWire extends Wire implements Drawable, GOutSignal, GInSignal {
	private Color color = Color.BLUE;
	private Color selColor = Color.RED;
	private Color relColor = Color.BLUE;
	
   private Line2D.Double line;
   private Rectangle2D.Double in;
   private Rectangle2D.Double out;
   private Rectangle2D.Double selectedEnd;
   private DigitalCircuit circuit;
   private static BasicStroke stroke;
   /**
    * Main constructor
    * @param pos Point2D.Double
    * @param c DigitalCircuit
    */
   public GWire(Point2D.Double pos, DigitalCircuit c){
      circuit = c;
      in = new Rectangle2D.Double( pos.x , pos.y , 10, 10 );
      out = new Rectangle2D.Double( pos.x + 80, pos.y , 10 , 10 ); 
      line = new Line2D.Double(in.x + 5, in.y + 5, out.x + 5, out.y + 5);
      selectedEnd = null;
      stroke = new BasicStroke(2);
   }
   /**
    * Constructor for the OpenFile class
    * This draw a wire with the input coordinates and the output coordinates
    * @param inx int
    * @param iny int 
    * @param outx int
    * @param outy int 
    * @param c DigitalCircuit
    */
   public GWire(int inx, int iny, int outx, int outy, DigitalCircuit c){
	      circuit = c;
	      in = new Rectangle2D.Double( inx , iny , 10, 10 );
	      out = new Rectangle2D.Double( outx , outy , 10 , 10); 
	      line = new Line2D.Double(in.x + 5, in.y + 5, out.x + 5, out.y + 5);
	      selectedEnd = in;
	      stroke = new BasicStroke(2);
	   }
   
   public Rectangle2D.Double getInPos(){
	   return in;
   }
   public Rectangle2D.Double getOutPos(){
	   return out;
   }
   
   public void draw (Graphics2D g){
      g.setColor(color);
      g.setStroke(stroke);
      g.draw(line);
      if (hasConnectedSource())
        g.fill(in);
      else
        g.draw(in);
      g.setColor(Color.GREEN);
      g.draw(out);
   }
   public boolean contains (Point p){
     boolean result=false;
     if (result = in.contains(p))
        selectedEnd=in;
     else if (result = out.contains(p))
             selectedEnd = out;
          else
             selectedEnd = null;
     return result;
   }
   public Shape getGOutput() {
     return out;
   }
   public Shape getGInput() {
     return in;
   }
   public void dragIn(int dx, int dy) {
      if (selectedEnd==null) return;
      selectedEnd.x+=dx;
      selectedEnd.y+=dy;
      line.setLine(in.x + 5, in.y + 5, out.x + 5, out.y + 5);
   }
   public void flip() {
   }
   public void setSelected() {
     color = selColor;
   }
   public void setReleased() {
     color = relColor;
   }
   public void setSelColor(Color c){
	   selColor = c;
   }
   public void setRelColor(Color c){
	   relColor = c;
   }
   /**
    * Set delay..
    * Not usable
    */
   public void setGDelay(float d){
	   
   }
   
   public void tryToConnect(){
      if (selectedEnd == in) {
         GOutSignal source = circuit.findSource(in);
         if (source !=null) {
        	//s.connectSound();
            source.connectOutput(this);
            Rectangle2D bound = source.getGOutput().getBounds();
            in.x = bound.getX();
            in.y = bound.getY();
         } else{
        	 disconnectInput();
        	 //s.disconnectSound();

         }
                     
      } else if (selectedEnd == out) { 
    	  GInSignal drain = circuit.findDrain(out);
    	  if (drain !=null) {
    //		  s.connectSound();
    		  drain.connectInput(this);
    		  Rectangle2D bound = drain.getGInput().getBounds();
    		  out.x = bound.getX();
    		  out.y = bound.getY();
    	  } else{ 
    		  disconnectAllOutput();
    	//	  s.disconnectSound();
    	  }
      }
         
      line.setLine(in.x + 5, in.y + 5, out.x + 5, out.y + 5);
   }
   /**
    * This reconnect the new wires add from the OpenFile class 
    */
   public void forceConnect(){
	          GOutSignal source = circuit.findSource(in);
	          if (source !=null) {
	      
	             source.connectOutput(this);
	             Rectangle2D bound = source.getGOutput().getBounds();
	             in.x = bound.getX();
	             in.y = bound.getY();
	          } else{
	         	 disconnectInput();

	          }
	                      
	     	  GInSignal drain = circuit.findDrain(out);
	     	  if (drain !=null) {

	     		  drain.connectInput(this);
	     		  Rectangle2D bound = drain.getGInput().getBounds();
	     		  out.x = bound.getX();
	     		  out.y = bound.getY();
	     	  } else{ 
	     		  disconnectAllOutput();
	     	  }
	       
	          
	       line.setLine(in.x + 5, in.y + 5, out.x + 5, out.y + 5);
	    }
   
   
@Override
public Path2D.Double getPos() {
	// TODO Auto-generated method stub
	return null;
}

@Override
public float getGDelay() {
	// TODO Auto-generated method stub
	return 0;
}
@Override
public int getIdDraw() {
	// TODO Auto-generated method stub
	return 0;
}
@Override
public void graph() {
	// TODO Auto-generated method stub
	
}

}