import java.awt.geom.Point2D;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Scanner;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

/**
 * Class OpenFile to read .dc file with the position of the gates
 * @author Marcio B
 *
 */
public class OpenFile {
	private ArrayList<Drawable> comp;
	private BufferedReader br;
	private FileReader fr;
	private Simulator sim;
	private DigitalCircuit digitalc;
	private JFrame frame;
	/**
	 * Constructor of OpenFile class
	 * @param filereader FileReader
	 * @param s Simulator
	 * @param dc DigitalCircuit
	 * @param f JFrame
	 */
	public OpenFile(FileReader filereader, Simulator s, DigitalCircuit dc, JFrame f){
		comp = new ArrayList<Drawable>();
		br = null;
		fr = filereader;
		sim = s;
		digitalc = dc;
		frame = f;
		
	}
	/**
	 *  This method read .dc file and adds components to a local ArrayList<Drawable>
	 */
	public void addComp(){
		String compName = "default";
		int id = 0, x = 0, y = 0;
		int winx = 0, winy = 0, woutx = 0, wouty = 0;
		int d = 0 ;
		try{
		br = new BufferedReader(fr);
		String linea;
		while((linea = br.readLine()) != null){
			if(linea.compareTo("GATE") == 0){
				compName = br.readLine();
				if(compName.compareTo("Wire") == 0){
					winx = Integer.parseInt(br.readLine())-5;
					winy = Integer.parseInt(br.readLine())-5;
					woutx = Integer.parseInt(br.readLine())-5;
					wouty = Integer.parseInt(br.readLine())-5;
			
					comp.add(new GWire(winx, winy, woutx, wouty, digitalc));
					
					System.out.println(compName +" "+ winx +" "+ winy +" "+ woutx +" "+ wouty);
				
				}else{
					id = Integer.parseInt(br.readLine());
					x = Integer.parseInt(br.readLine());
					y = Integer.parseInt(br.readLine());
					d = Integer.parseInt(br.readLine());
					System.out.println(compName +" "+ id +" "+ x +" "+ y +" "+ d);
				
					if(compName.compareTo("And") == 0)
						comp.add(new GAndGate(new Point2D.Double(x-100,y-20), sim));
						
					if(compName.compareTo("Nor") == 0)
						comp.add(new GNorGate(new Point2D.Double(x-100,y-20), sim));
					
					if(compName.compareTo("Or") == 0)
						comp.add(new GOrGate(new Point2D.Double(x-80, y-20), sim));

					if(compName.compareTo("Nand") == 0)
						comp.add(new GNandGate(new Point2D.Double(x-100,y-20), sim));

					if(compName.compareTo("Inverter") == 0)
						comp.add(new GInverterGate(new Point2D.Double(x-16,y), sim));
				
					if(compName.compareTo("Xor") == 0)
						comp.add(new GXorGate(new Point2D.Double(x-100,y-20), sim));
					
					if(compName.compareTo("So") == 0){
				    	  Scanner data=null;
				    	  JFileChooser chooser = new JFileChooser();
				    	  FileNameExtensionFilter filter = new FileNameExtensionFilter(".txt data files", "txt");
				    	  chooser.setFileFilter(filter);
				    	  int returnVal = chooser.showOpenDialog(null);
				    	  if(returnVal == JFileChooser.APPROVE_OPTION) {
				    		  try {
				    			  data= new Scanner(chooser.getSelectedFile());
				    		  } catch(FileNotFoundException ef) {
				    			  JOptionPane.showMessageDialog(null, "Can't open the file.\nPlease try again. ","Error", JOptionPane.ERROR_MESSAGE);  
				    			  return;
				    		  }
				    		  if(!data.hasNext()){
				  				JOptionPane.showMessageDialog(null, "File is EMPTY.\nPlease try again. ","Error", JOptionPane.ERROR_MESSAGE);
				  			}else
				  				comp.add(new GSource(new Point2D.Double(x-32,y-25), data, sim));
				    		 
				    	  }
						
					}
						
					if(compName.compareTo("M") == 0){
					 	  PrintStream outDat = null;
				    	  JFileChooser fcho = new JFileChooser();
				    	  FileNameExtensionFilter filer = new FileNameExtensionFilter(".txt", "txt");
				    	  fcho.setFileFilter(filer);

				    	  int returnVe = fcho.showSaveDialog(null);
				    	  if(returnVe == JFileChooser.APPROVE_OPTION) {
				    		  try {
				    			  outDat = new PrintStream(fcho.getSelectedFile());
				    		  } catch(FileNotFoundException ef) {
				    			  JOptionPane.showMessageDialog(null, "Can't save the file.\nPlease try again. ","Error", JOptionPane.ERROR_MESSAGE);

				    			  return;
				    		  }
				    		  //circuit.addMeter(outDat,frame);
				    		    comp.add(new GMeter(new Point2D.Double(x-67, y-25), outDat, sim, frame, new GraphFrame()));
				    	  }
						
					}
				}
			}
		}

		}catch(Exception e){
			e.printStackTrace();
		}finally{
			try{
				if( null != fr)
					fr.close();
			
		}catch(Exception e2){
			e2.printStackTrace();
		}
			
			}
		

	
}
	/**
	 * This method return the local ArrayList<Drawable> 
	 * @return ArrayList<Drawable>
	 */
	public ArrayList<Drawable> getComponents(){
	
		return comp;
	}
	
}
